/**
 *
 */
package com.sas.hadoopvalidator

import java.io.BufferedReader;
import java.io.FileNotFoundException;
import java.io.FileReader;
import java.io.IOException;
import java.io.File;
import java.util.regex.*;

import com.sas.ssn.DynamicDialogs
import com.sas.ssn.Globals
import com.sas.ssn.InstallData
import com.sas.ssn.Logging
import com.sas.ssn.Utils
import com.sas.ssn.validate.ValidationResult
import com.sas.ssn.validate.Validator
import com.sas.ssn.WizardData
import com.sas.util.SasPasswordString;

import com.sas.hadoop.*

import java.net.InetAddress;
import java.net.MalformedURLException;
import java.net.UnknownHostException;
import java.net.URISyntaxException;
import java.net.URL;

/**
 * @author Chris Choi
 *
 */
 class MultiNodesValidator extends Validator{

      final String PRODUCT_CODE = "hadoop"
      public static final String MSG_OOZIENODE_CONNECTION_FAILED = "access.hadoop.config.cluster.oozienode.connection.failed"
      public static final String MSG_DOMAIN_NAME_NOT_VALID = "access.hadoop.config.domain.name.not.valid"
      public static final String MSG_HOST_NAME_NOT_VALID = "access.hadoop.config.host.name.not.valid"
      public static final String MSG_HIVENODE_CONNECTION_FAILED  = "access.hadoop.config.cluster.hivenode.connection.failed"
      public static final String MSG_CLUSTER_MANAGER_CONNECTION_FAILED ="access.hadoop.config.cluster.manager.connection.failed"
      public static final String MSG_CLUSTER_SERVICES_NOT_STARTED = "access.hadoop.config.cluster.services.not.started"
      public static final String MSG_STRACE_NOT_INSTALLED = "access.hadoop.prereq.package.strace.not.installed"
      public static final String MSG_HADOOP_NOT_IN_PATH   = "access.hadoop.prereq.hadoop.not.defined.in.path"
      public static final String MSG_HIVE_NOT_IN_PATH     = "access.hadoop.prereq.hive.not.defined.in.path"
      public static final String MSG_PIG_NOT_IN_PATH      = "access.hadoop.prereq.pig.not.defined.in.path"
      public static final String MSG_HBASE_NOT_IN_PATH    = "access.hadoop.prereq.hbase.not.defined.in.path"
      public static final String MSG_YARN_NOT_IN_PATH    = "access.hadoop.prereq.yarn.not.defined.in.path"
      public static final String MSG_COMMAND_NOT_IN_PATH  = "access.hadoop.prereq.command.not.defined.in.path"
      public static final String MSG_HDFS_USER_DIRECTORY_NOT_WRITABLE  = "hadoop.prereq.hdfs.user.directory.not.writable"
      public static final String MSG_HIVE_KERBEROS_TICKET_NOT_READY = "access.hadoop.hivenode.kerberos.ticket.not.ready"
      public static final String MSG_HIVE_NOT_ACTIVE = "access.hadoop.prereq.hive.not.active"
      public static final String MSG_HIVE_INSTANCE_NOT_CREATED = "access.hadoop.hive.service.not.instantiated"

      final String HADOOPTRACER            = "hadooptracer.py"
      final String CONFIG_FILE_DIR         = "conf"
      final String JAR_FILE_DIR            = "lib"

      private static String  sConfigFilesDirName   = null
      private static String  sJarFilesDirName      = null
      private static String  sUserHomeDirName      = null
      private static String  sHiveNodeRepository   = null
      private static String  sOozieNodeRepository   = null
      private static String  sHadoopTracerFilePath = null
      String clusterNameCombo             = null
      String sFQDNHostName                = null
      String sIPAddress                   = null
      private static String sUserOwnDir, sTestDir, distribution, distributionName, SSHAccountPassword, SSHPrivateKeyFile, sOutPut
      private static String sUserDir = "/user"

      def String getProductCodeProperty(){
           return getDialogData(PRODUCT_CODE)
      }

      def ValidationResult validateData(){

          ConfigUtils cu = new ConfigUtils()

          ValidationResult result = new ValidationResult()

          def productCode = getProductCodeProperty()

          File sasHadoopPropsFile = new File (com.sas.hadoop.Globals.g_configPropertyFileName)
          if (com.sas.hadoop.Globals.g_hadoopVendor == "other" && sasHadoopPropsFile.exists()){
               try{
                    if(sasHadoopPropsFile.delete()){
                	    Logging.log.info(com.sas.hadoop.Globals.g_configPropertyFileName + " is deleted.")
                    }else{
                	    Logging.log.info("Delete operation is failed.")
                    }
                    cu.createPropsFile()
                }catch(Exception e){
                    e.printStackTrace();
       	        }
           }
           else {
                if (!sasHadoopPropsFile.exists()) {
                   try {
                       cu.createPropsFile()
                   } catch (IOException e){
                     Logging.log.info ("Unable to create the configuration properties file: " + sasHadoopPropsFile)
                     result.setResult(ValidationResult.RESULT_TYPE_ERROR, cu.MSG_BASEDIR_NOT_EXIST, sasHadoopPropsFile, cu.getBaseDir())
          	     return result
                     //Logging.logStackTrace (e)
                   }
                }
           }

           com.sas.hadoop.Globals.g_hivenodeAdminName = Globals.g_installData.getProperty("access.hadoop.config.cluster.ssh.credentials.admin", null);
    	   cu.addProp("hadoop.cluster.hivenode.ssh.account",com.sas.hadoop.Globals.g_hivenodeAdminName)

           if (com.sas.hadoop.Globals.g_hivenodeCredentialType == "privatekey"){
               com.sas.hadoop.Globals.g_SSHKeyFile = Globals.g_installData.getProperty("access.hadoop.config.cluster.ssh.credentials.privatekey", null);
               com.sas.hadoop.Globals.g_SSHport = Globals.g_installData.getProperty("access.hadoop.config.cluster.ssh.credentials.portnumber", null);
               cu.addProp("hadoop.cluster.manager.ssh.prvkey", com.sas.hadoop.Globals.g_SSHKeyFile)
               cu.addProp("hadoop.cluster.manager.ssh.port", com.sas.hadoop.Globals.g_SSHport)
               SSHAccountPassword = "{sas002}DA9A0A5C20629B7F34D2C88A165E5530"
               Logging.log.info ("Private key file: " + com.sas.hadoop.Globals.g_SSHKeyFile)
               Logging.log.info ("SSH port number: " + com.sas.hadoop.Globals.g_SSHport)
           }
           else if (com.sas.hadoop.Globals.g_hivenodeCredentialType == "password"){
               SSHAccountPassword = Globals.g_installData.getProperty("access.hadoop.config.cluster.ssh.credentials.password", null)
               com.sas.hadoop.Globals.g_hivenodeAdminPassword = SSHAccountPassword
               //Logging.log.info ("SSHAccountPassword: " + SSHAccountPassword)
               cu.addProp("hadoop.cluster.hivenode.ssh.password", SSHAccountPassword)
           }

    	   //cu.addProp("hadoop.cluster.distribution.name",ClusterMgrPortNumberValidator.hadoopVendor)
    	   Logging.log.info ("com.sas.hadoop.Globals.g_hivenodeAdminName is " + com.sas.hadoop.Globals.g_hivenodeAdminName)

    	   //Test see if we can Connect to the Hive Node
           int rcTest = 0;
           try {
                Connect connect = new com.sas.hadoop.Connect(com.sas.hadoop.Globals.g_hivenodeAdminName,
                                                             com.sas.hadoop.Globals.g_hivenodeHostName,
                                                             SasPasswordString.decode(SSHAccountPassword))

                 if (com.sas.hadoop.Globals.g_hivenodeCredentialType != "privatekey"){
                     Logging.log.info ("Password is used.")
                     rcTest = connect.establish (com.sas.hadoop.Globals.g_hivenodeAdminName,
                                                                com.sas.hadoop.Globals.g_hivenodeHostName,
                                                                SasPasswordString.decode(SSHAccountPassword),
                                                                true)
                 }
                 else {
                      Logging.log.info ("Private key is used.")
                      rcTest = connect.establish (com.sas.hadoop.Globals.g_hivenodeAdminName,
                                                                  com.sas.hadoop.Globals.g_hivenodeHostName,
                                                                  SasPasswordString.decode(SSHAccountPassword),
                                                                  true)
                 }
 
                 if (connect.bConnected){
                     Logging.log.info ("Connection to the hive node " + com.sas.hadoop.Globals.g_hivenodeHostName + " was successful." )
                     try {                     
                          if(connect.remoteExec("which strace")==false){
                             Logging.log.severe("The Linux package 'strace' is not installed. You must install the package strace using the command 'yum install strace'")
                             result.setResult(ValidationResult.RESULT_TYPE_ERROR, MSG_STRACE_NOT_INSTALLED)
                             return result
                          }
                          else {
                              Logging.log.info("Validated that the Linux package 'strace' is installed.")
                          }
                
                          if(connect.remoteExec("which hadoop")==false){
                             Logging.log.severe("'hadoop' must be defined in the system environment variable PATH. You must add 'hadoop' in your cluster's default PATH variable.")
                             result.setResult(ValidationResult.RESULT_TYPE_ERROR, MSG_HADOOP_NOT_IN_PATH)
                             return result
                          }
                          else {
                               Logging.log.info("Validated that hadoop is defined in the system environment variable PATH.")
                          }
                
                          if(connect.remoteExec("which hive")==false){
                             Logging.log.severe("'hive' must be defined in the system environment variable PATH. You must add 'hive' in your cluster's default PATH variable.")
                             result.setResult(ValidationResult.RESULT_TYPE_ERROR, MSG_HIVE_NOT_IN_PATH)
                             return result
                          }                          
                          else {
                               Logging.log.info("Validated that hive is defined in the system environment variable PATH.")
                          }
                          
                          /*int rcCheck = 0
                          MultiNodesValidator mnv = new MultiNodesValidator()
                          rcCheck = mnv.runPrereqcheck(connect)
                          if (rcCheck == 0)
                             Logging.log.info ("Validated that all prerequisites is configured correctly. The return code is " + rcCheck)
                          else {
                              if (rcCheck == -27){
                                  result.setResult(ValidationResult.RESULT_TYPE_ERROR, MSG_HDFS_USER_DIRECTORY_NOT_WRITABLE, sUserOwnDir)
                                  return result
                              }
                              if (rcCheck == -30){
                                  result.setResult(ValidationResult.RESULT_TYPE_ERROR, MSG_HIVE_INSTANCE_NOT_CREATED)
                                  return result
                              }
                              if (rcCheck == -31){
                                  result.setResult(ValidationResult.RESULT_TYPE_ERROR, MSG_HIVE_NOT_ACTIVE)
                                  return result
                              }
                          }*/ 

                         sOutPut = connect.remoteCmdOutput("hadoop fs -ls /user")
                         //sUserOwnDir = sUserDir + "/" + com.sas.hadoop.Globals.g_hivenodeAdminName
                         //Logging.log.info(sOutPut)
                         //if (sOutPut.contains(sUserOwnDir)){
                         //    Logging.log.info ("The HDFS user home directory " + sUserOwnDir + " exists.")
                         //}
                         //else {
                               if (sOutPut.contains("GSSException")){
                                   result.setResult(ValidationResult.RESULT_TYPE_ERROR, MSG_HIVE_KERBEROS_TICKET_NOT_READY)
                                   return result

                               }
                         //      else {
                         //           Logging.log.severe("The HDFS user home directory " + sUserOwnDir + " does not exist.")
                         //           result.setResult(ValidationResult.RESULT_TYPE_ERROR, MSG_HDFS_USER_DIRECTORY_NOT_WRITABLE, sUserOwnDir)
                         //           return result
                         //       }
                         //} 
                     }
                     catch (Exception ef){
           	            Logging.logStackTrace(ef)
                     }
                 }
                 else if (rcTest ==-2) {
                          result.setResult(ValidationResult.RESULT_TYPE_ERROR, MSG_HIVENODE_CONNECTION_FAILED)
                          return result
                 }
                 else if (rcTest ==-1) {
                          result.setResult(ValidationResult.RESULT_TYPE_ERROR, MSG_HIVENODE_CONNECTION_FAILED)
                          return result
                 }
           }
           catch (com.jcraft.jsch.JSchException je){
                  result.setResult(ValidationResult.RESULT_TYPE_ERROR, MSG_HIVENODE_CONNECTION_FAILED)
                  Logging.logStackTrace(je)
                  return result
           }
           catch (Exception e) {
                  result.setResult(ValidationResult.RESULT_TYPE_ERROR, MSG_HIVENODE_CONNECTION_FAILED)
                  Logging.logStackTrace(e)
                  return result
           }

           //initialize the Client Directories with the default values
           Globals.g_installDefaults.setDefault("access.hadoop.config.directory.base",cu.sasHadoopBaseDir)
           Globals.g_installDefaults.setDefault("access.hadoop.config.props.file", com.sas.hadoop.Globals.g_configPropertyFileName)
           Globals.g_installDefaults.setDefault("access.hadoop.config.class.path",ClusterMgrPortNumberValidator.requiredJarListPath)
           com.sas.hadoop.Globals.g_configfilePath = Utils.joinPath (cu.sasHadoopBaseDir, CONFIG_FILE_DIR)
           com.sas.hadoop.Globals.g_jarPath  = Utils.joinPath (cu.sasHadoopBaseDir, JAR_FILE_DIR)
           Globals.g_installDefaults.setDefault("access.hadoop.config.directory.configfiles", com.sas.hadoop.Globals.g_configfilePath)
           Globals.g_installDefaults.setDefault("access.hadoop.config.directory.jarfiles",com.sas.hadoop.Globals.g_jarPath)

      	return result
   }

   public int runPrereqcheck(Connect connect){
         ConfigUtils cu = new ConfigUtils()
         int checkPrereq = 0
         String sOutput = null
         String sOutPut, delims, sTemp, sMapReduceDir
         sUserOwnDir = sUserDir + "/" + com.sas.hadoop.Globals.g_hivenodeAdminName
         sOutPut = connect.remoteCmdOutput("hive -e 'set -v'|grep dfs.user.home.dir.prefix")
         Logging.log.info (sOutPut)
         if (sOutPut.contains("java.lang.RuntimeException") && sOutPut.contains("SessionHiveMetaStoreClient")){
             return -30
         }
         if (sOutPut.contains("java.lang.RuntimeException") && sOutPut.contains("java.net.ConnectException")){
             return -31
         }

         if (sOutPut.contains("dfs.user.home.dir.prefix")){
             delims = "dfs.user.home.dir.prefix="
             String[] tokens = sOutPut.split(delims)
             for (int i = 0; i < tokens.length; i++){
                 sTemp = tokens[i]
                 if (sTemp !=""){
                     sUserDir = sTemp.trim()
                 }
             }

             if (sUserDir.contains("|")){
                 Scanner sc = new Scanner(sUserDir);
                 String line = sc.nextLine();
                 delims = " "
                 tokens = line.split(delims)
                 sTemp = tokens[0]
                 if (sTemp !=""){
                     sUserDir = sTemp.trim()
                 }
                 //Logging.log.info ("sUserDir is " + sUserDir)
             }

             sMapReduceDir = sUserDir + "/" + "mapred"
             sUserOwnDir = sUserDir + "/" + com.sas.hadoop.Globals.g_hivenodeAdminName
             sOutPut = connect.remoteCmdOutput("hadoop fs -ls " + sUserOwnDir)
             Logging.log.info (sOutPut)
             if (sOutPut.contains("PriviledgedActionException") && sOutPut.contains("GSSException")){
                 checkPrereq = -28
             }

             //Logging.log.info("The output of the command: " + sOutPut)
             /**
             if(connect.remoteExec("hadoop fs -ls " + sUserOwnDir) == false){
                sOutPut = connect.remoteCmdOutput("hadoop fs -ls " + sUserDir)
                Logging.log.info (sOutPut)
                if (sOutPut.contains("PriviledgedActionException") && sOutPut.contains("GSSException")){
                    checkPrereq = -28
                }

                if (!sOutPut.contains("drwxrwxrwx   - mapred")){
                    Logging.log.severe("The user mapred account does not have a write permission to the HDFS file location /user.")
                    checkPrereq = -27
                }
             }
             else {
                 Logging.log.info ("The HDFS user home directory " + sUserOwnDir + " exists.")
                 sTestDir = sUserOwnDir + "/" + "test1"
                 Logging.log.info ("Testing to see if the HDFS user home directory " + sTestDir + " can be created.")
                 if(connect.remoteExec("hadoop fs -mkdir " + sTestDir)==false){
                     Logging.log.severe("The user home directory " + sUserOwnDir +  " is not writable.")
                     checkPrereq = -27
                 }
                 else{
                     connect.remoteExec("hadoop fs -rmr -skipTrash " + sTestDir)
                     Logging.log.info("The user home directory " + sUserOwnDir +  " is writable.")
                 }
              }*/
         }
         else {
             sOutPut = connect.remoteCmdOutput("hadoop fs -ls /user")

             //Logging.log.info("The output of the command: " + sOutPut)
             if (sOutPut.contains(sUserOwnDir)){
                 Logging.log.info ("The HDFS user home directory " + sUserOwnDir + " exists.")
                 sTestDir = sUserOwnDir + "/" + "test1"
                 Logging.log.info ("Testing to see if the HDFS user home directory " + sTestDir + " can be created.")
                 if(connect.remoteExec("hadoop fs -mkdir " + sTestDir) == false){
                     Logging.log.severe("The user home directory " + sUserOwnDir +  " is not writable.")
                     checkPrereq = -27
                 }
                 else{
                     connect.remoteExec("hadoop fs -rmr -skipTrash " + sTestDir)
                     Logging.log.info("The user home directory " + sUserOwnDir +  " is writable.")
                 }
             }
             else {
                 if (sOutPut.contains("GSSException")){
                     checkPrereq = -28
                 }
                 else {
                     Logging.log.severe("The HDFS user home directory " + sUserOwnDir + " does not exist.")
                     checkPrereq = -27
                 }
             }
         }
         return checkPrereq
       }
}
